import { membership } from "../../entity/membership";
import { MembershipViewModel } from "../../viewmodel/admin/membership.models";

export default abstract class MembershipFactory {
  /**
   * @description map record to membership
   * @param {membership} record
   * @returns {MembershipViewModel}
   */
  public static mapToSingle(record: membership): MembershipViewModel {
    return {
      id: record.id,
      start: record.start,
      end: record.end,
      terminationReason: record.terminationReason,
      status: record.status.status,
      statusId: record.status.id,
    };
  }

  /**
   * @description map records to membership
   * @param {Array<membership>} records
   * @returns {Array<MembershipViewModel>}
   */
  public static mapToBase(records: Array<membership>): Array<MembershipViewModel> {
    return records.map((r) => this.mapToSingle(r));
  }
}
